#include <gtk/gtk.h>
#include <string.h>
#include "attach.h"
#include "support.h"
#include "question.h"

extern GtkBuilder *builder;



gboolean
on_enter_notify_event                  (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	GdkCursor* cursor;
	cursor = gdk_cursor_new(GDK_HAND1);
	gdk_window_set_cursor(widget->window, cursor);
	gdk_cursor_destroy(cursor);
	
	return FALSE;
}

gboolean
on_leave_notify_event                  (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	GdkCursor* cursor;
	cursor = gdk_cursor_new(0);
	gdk_window_set_cursor(widget->window, cursor);
	gdk_cursor_destroy(cursor);

	return FALSE;
}


gboolean
on_button_press_event			(GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	GtkWidget *widget_button;
	gchar warning_msg[256] = "\0";

	attach_num --;
	add_attach_num --;

	g_sprintf (warning_msg, _("Attach limit is %d used %d"), (screenshot_num + attachable_num), attach_num);
	widget_button = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach"));	
	gtk_widget_set_sensitive (widget_button, TRUE);	
	gtk_widget_set_tooltip_text (widget_button, warning_msg);

	widget_button = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_any"));	
	gtk_widget_set_sensitive (widget_button, TRUE);		
	gtk_widget_set_tooltip_text (widget_button, warning_msg);

	widget_button = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_pause"));	
	gtk_widget_set_sensitive (widget_button, TRUE);		
	gtk_widget_set_tooltip_text (widget_button, warning_msg);

	gtk_widget_destroy (user_data);
	return FALSE;
}


void
add_attach		(gchar *filename)
{
	GtkWidget *hbox;
	GtkWidget *label_shortname;
	GtkWidget *label_delete;
	GtkWidget *label_hide_full_name;
	GtkWidget *event_box;
	GtkWidget *box_labels;
	gchar short_name[512];


	box_labels = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_labels"));
	hbox = gtk_hbox_new (FALSE, 5);
	gtk_box_pack_start (GTK_BOX (box_labels), hbox, FALSE, FALSE, 0);

	g_sprintf (short_name, "    %s", filename+search_last_slash(filename));
	if (short_name [strlen(short_name)-1] == '\n')
	{
		short_name [strlen(short_name)-1] = '\0';
	}
	label_shortname = gtk_label_new (short_name);
	gtk_box_pack_start (GTK_BOX (hbox), label_shortname, FALSE, FALSE, 0);

	label_hide_full_name = gtk_label_new (filename);

	gtk_box_pack_start (GTK_BOX (hbox), label_hide_full_name, FALSE, FALSE, 0);
	gtk_widget_set_no_show_all (label_hide_full_name, TRUE);
	
	event_box = gtk_event_box_new ();
	gtk_box_pack_end (GTK_BOX (hbox), event_box, FALSE, FALSE, 0);
	g_signal_connect (event_box, "button_press_event",G_CALLBACK (on_button_press_event), hbox);
	g_signal_connect (event_box, "enter_notify_event",G_CALLBACK (on_enter_notify_event), hbox);
	g_signal_connect (event_box, "leave_notify_event",G_CALLBACK (on_leave_notify_event), hbox);

	label_delete = gtk_label_new ("");
	gtk_label_set_markup (GTK_LABEL (label_delete), _("<span fgcolor=\"#0000FF\" underline=\"single\">delete</span>"));
	gtk_container_add (GTK_CONTAINER (event_box), label_delete);

	attach_num ++;
	gtk_widget_show_all (hbox);
	gtk_widget_show_all (box_labels);
	gtk_widget_hide (label_hide_full_name);
	gtk_widget_show_all (hbox);
}


static void
for_each_attach 	(GtkWidget *child,
			 gpointer   client_data)
{
	if ( gtk_widget_get_visible (child) == FALSE)
	{
		g_fprintf (client_data, "[ATTACHMENT$]: %s\n", gtk_label_get_text (GTK_LABEL (child)));
	}
}


static void
for_each_hbox 			(GtkWidget *child,
			 	 gpointer   client_data)
{
	gtk_container_forall (GTK_CONTAINER (child),for_each_attach, client_data);
}


static void
for_each_hbox_destroy 		(GtkWidget *child,
				 gpointer   client_data)
{
	gtk_widget_destroy (child);
}


void
destroy_attach_widgets		 ()
{
	GtkWidget *widget;
	GtkWidget *box_labels;

	box_labels = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_labels"));

	gtk_container_forall (GTK_CONTAINER (box_labels),for_each_hbox_destroy, NULL);

	attach_num = 0;
}


void
read_attach			(FILE *filetowrite)
{
	GtkWidget *box_labels;

	box_labels = GTK_WIDGET (gtk_builder_get_object (builder, "vbox_labels"));

	gtk_container_forall (GTK_CONTAINER (box_labels), for_each_hbox,filetowrite);
}


FILE *
open_for_attachment (gint file_id)
{
	gchar filename[256];
	FILE *f;
	
	g_sprintf (filename,"%s/%s/store/%d", g_get_home_dir() , CACHE_DIR, file_id);
//	printf ("%s\n", filename);
	f = fopen (filename, "r");
	if (!f)
	{
		return 0;
	}

	return f;
}

void
close_attachment (FILE *file)
{
	fclose (file);
}

gint
get_attachment (FILE *file, gchar *out_name, gchar *out_fullname)
{
	ssize_t read;
	size_t len = 0;
	gchar *line = NULL;


	while ((read = getline(&line, &len, file)) != -1) 
	{
		if (g_strrstr (line, "[ATTACHMENT$]: "))
		{
			g_sprintf (out_name, "%s", line+search_last_slash (line));
			g_sprintf (out_fullname, "%s", line+15);
			g_free (line);
			return 1;
		}
	}

	if (line != NULL)
	{
		g_free (line);
	}

	return 0;
}

