#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <glib-object.h>

#include "callbacks.h"
#include "state.h"
#include "xml_work.h"
#include "shorthands.h"
#include "question.h"
#include "curfiletime.h"
#include "pthreads.h"
#include "attach.h"
#include "support.h"

gint
check_buffer (GtkTextBuffer* buffer)
{
	GtkTextIter iter_start, iter_end;
	gchar *text = NULL;
	GtkWidget *widget;
	GdkColor color;
	color.red = 65535;
	color.green = 26774;
	color.blue = 29601;

	gtk_text_buffer_get_start_iter (buffer, &iter_start);
	gtk_text_buffer_get_end_iter  (buffer, &iter_end);

	text = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buffer), &iter_start, &iter_end, 0);

	if (strlen (text) > 8192)
	{
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "textview1"));
		gtk_widget_modify_text (widget, GTK_STATE_NORMAL, &color);
		gtk_widget_set_tooltip_text (widget, _("Warning: char limit is 512!"));
	}
	else
	{
		gdk_color_parse ("black", &color);
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "textview1"));
		gtk_widget_modify_text (widget, GTK_STATE_NORMAL, &color);
		gtk_widget_set_tooltip_text (widget, "");
	}

	if (strlen (text) == 0)
	{
		return 0;
	}

	g_free (text);
	return 1;
}

void                
on_textbuffer1_changed			(GtkTextBuffer *buffer,
                      		   	gpointer       user_data)
{
	GtkWidget *widget;
	gint res;

	widget = GTK_WIDGET (gtk_builder_get_object (builder, "radiobutton2"));

	res = check_buffer (GTK_TEXT_BUFFER (buffer));
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
	{
		if (res == 0)
		{
			gtk_widget_set_tooltip_text (ForwardButton, _("Please enter a comment"));
			gtk_widget_set_sensitive (ForwardButton, FALSE);
		}
		else
		{
			gtk_widget_set_tooltip_text (ForwardButton, "");
			gtk_widget_set_sensitive (ForwardButton, TRUE);
		}
	}
}


G_MODULE_EXPORT void
on_window1_destroy                	(GtkObject       *object,
                                        gpointer         user_data)

{
	if (started == 0)
	{
		clean_after_send ();
	}
	
	if (started != 100)
	{
		write_question_answer ();
	}
	
	gtk_main_quit();
}


G_MODULE_EXPORT void
on_button_exit_clicked			(GtkObject       *object,
                                        gpointer         user_data)
{
	if (started == 0)
	{
		clean_after_send ();
	}

	if (started != 100)
	{

		write_question_answer ();
	}
	gtk_main_quit();
}


G_MODULE_EXPORT void
on_button_forward_clicked               (GtkObject       *object,
                                        gpointer         user_data)
{
	GtkBuilder *builder_local;
	GtkWidget *spinner_window;

	switch (cur_state)
	{
		case WELCOME:
		    set_state (REGISTER);
		    check_reg_form ();
		break;
		case REGISTER:
			spinner_window = SpinnerWindow (_("Registration and getting testplans. Please wait"));
		    	g_thread_create (pthread_register, spinner_window, FALSE, NULL);
		break;
		case GETTESTPLAN:
			spinner_window = SpinnerWindow (_("Getting testplans. Please wait"));
			g_thread_create (pthread_get_testplans, spinner_window, FALSE, NULL);
		break;
		case TEST:
			if (!get_next_question ())
			{
					
				set_state (SEND);
			}
			gtk_widget_show (BackButton);
		break;
	}
}

G_MODULE_EXPORT void
on_button_back_clicked               (GtkObject       *object,
                                      gpointer         user_data)
{

	switch (cur_state)
	{
		case REGISTER:
		    set_state (WELCOME);
		break;
		
		case TEST:
			if (get_prev_question ())
			{
			        gtk_widget_hide (BackButton);
		                gtk_widget_show (ForwardButton);
			}
		break;
		
		case SEND:
			set_state_test_after_send ();
		break;
	}
}

gint
is_skipped ()
{
	GtkWidget * widget;

	widget = GTK_WIDGET (gtk_builder_get_object (builder, "radiobutton3"));
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
	{
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach"));
		gtk_widget_set_sensitive (widget, FALSE);
		gtk_widget_set_tooltip_text (widget, _("to activate this button, please answer on question"));
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_pause"));
		gtk_widget_set_sensitive (widget, FALSE);
		gtk_widget_set_tooltip_text (widget, _("to activate this button, please answer on question"));
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_any"));
		gtk_widget_set_sensitive (widget, FALSE);
		gtk_widget_set_tooltip_text (widget, _("to activate this button, please answer on question"));
		answered = 0;
	}
	else
	{
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach"));
		gtk_widget_set_sensitive (widget, TRUE);
		gtk_widget_set_tooltip_text (widget, "");
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_pause"));
		gtk_widget_set_sensitive (widget, TRUE);
		gtk_widget_set_tooltip_text (widget, "");
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_any"));
		gtk_widget_set_sensitive (widget, TRUE);
		gtk_widget_set_tooltip_text (widget, "");
		answered = 1;
	}

	return 0;
}

G_MODULE_EXPORT void
on_radiobutton1_toggled			(GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_widget_set_sensitive (ForwardButton, TRUE);
	is_skipped ();
}

G_MODULE_EXPORT void
on_radiobutton2_toggled			(GtkObject       *object,
                                        gpointer         user_data)
{
	gint res;
	is_skipped ();

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (object)))
	{
		res = check_buffer (gtk_builder_get_object (builder, "textbuffer1"));

		if (res == 0)
		{
			gtk_widget_grab_focus (GTK_WIDGET (gtk_builder_get_object (builder, "textview1")));
			gtk_widget_set_tooltip_text (ForwardButton, _("Please enter a comment"));
			gtk_widget_set_sensitive (ForwardButton, FALSE);
		}
		else
		{
			gtk_widget_set_tooltip_text (ForwardButton, "");
			gtk_widget_set_sensitive (ForwardButton, TRUE);
		}
	}
	else
	{
		gtk_widget_set_tooltip_text (ForwardButton, "");
		gtk_widget_set_sensitive (ForwardButton, TRUE);
	}
}

G_MODULE_EXPORT void
on_radiobutton3_toggled			(GtkObject       *object,
                                        gpointer         user_data)
{
	is_skipped ();
	
}

G_MODULE_EXPORT void
on_button_send_clicked			(GtkObject       *object,
                                        gpointer         user_data)
{
	GtkBuilder *builder_local;
	GtkWidget *spinner_window;

	spinner_window =  SpinnerWindow (_("Gathering report and sending. Please wait"));
	g_thread_create (pthread_send_report, spinner_window, FALSE, NULL);
}


G_MODULE_EXPORT void
on_button_attach_clicked		(GtkObject       *object,
                                        gpointer         user_data)
{
	time_t result;
	gchar filename[256] = "/0", time_str[128], warning_msg[256]="\0";
	GdkPixbuf *screenshot;
	GtkWidget *window, *widget;

	window = GTK_WIDGET (gtk_builder_get_object (builder, "window1"));
	gtk_window_iconify (GTK_WINDOW (window));
	screenshot = get_screenshot ();
	gtk_window_deiconify (GTK_WINDOW (window));
	result = time(NULL);
	strftime (time_str, 128, "%FT%T", localtime(&result));
	g_sprintf (filename, "%s/%s/screenshots/%d_%s.jpeg", g_get_home_dir (), CACHE_DIR, add_attach_num ++, time_str);
	add_attach (filename);
	gdk_pixbuf_save (screenshot, filename, "jpeg", NULL,  "quality", "100", NULL);	


	if (add_attach_num == (screenshot_num + attachable_num))
	{
		g_sprintf (warning_msg, _("Attach limit is reached"));
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach"));	
		gtk_widget_set_sensitive (widget, FALSE);	
		gtk_widget_set_tooltip_text (widget, warning_msg);	

		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_any"));	
		gtk_widget_set_sensitive (widget, FALSE);		
		gtk_widget_set_tooltip_text (widget, warning_msg);	

		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_pause"));	
		gtk_widget_set_sensitive (widget, FALSE);		
		gtk_widget_set_tooltip_text (widget, warning_msg);	
	}
	else
	{
		g_sprintf (warning_msg, _("Attach limit is %d used %d"), (screenshot_num + attachable_num), add_attach_num);
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach"));	

		gtk_widget_set_tooltip_text (widget, warning_msg);	
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_any"));	

		gtk_widget_set_tooltip_text (widget, warning_msg);	
		widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_attach_pause"));	

		gtk_widget_set_tooltip_text (widget, warning_msg);	
	}

}


G_MODULE_EXPORT void
on_entry_name_changed			(GtkObject       *object,
                                        gpointer         user_data)
{
	check_reg_form ();
}


G_MODULE_EXPORT void
on_entry_fam_changed			(GtkObject       *object,
                                        gpointer         user_data)
{
	check_reg_form ();
}


G_MODULE_EXPORT void
on_entry_mail_changed			(GtkObject       *object,
                                        gpointer         user_data)
{
	check_reg_form ();
}


G_MODULE_EXPORT void
on_entry_comment_changed		(GtkObject       *object,
                                        gpointer         user_data)
{
	check_reg_form ();
}


G_MODULE_EXPORT void
on_button_attach_pause_clicked		(GtkObject       *object,
                                        gpointer         user_data)
{
	GtkBuilder *builder_local;
	GtkWidget *spinner_window;
	gchar text[256];

	g_sprintf (text, _("Waiting for screenshot %d sec"), time_pause);

	spinner_window = SpinnerWindow (text);
	g_thread_create (pthread_attach_pause, spinner_window, FALSE, NULL);
}


G_MODULE_EXPORT void
on_button_attach_any_clicked		(GtkObject       *object,
                                        gpointer         user_data)
{
	GtkWidget *dialog;

	CreateFileChooser (_("Attach file"), "");
}

G_MODULE_EXPORT void 
on_label_welcome_size_allocate		(GtkWidget     *label,
            			 GtkAllocation *allocation,
                		 gpointer       data )

{
	gtk_widget_set_size_request (label, allocation->width - 20, -1);
}


G_MODULE_EXPORT void 
on_label_send_pad_size_allocate	(GtkWidget     *label,
            			 GtkAllocation *allocation,
                		 gpointer       data )

{
	gtk_widget_set_size_request (label, allocation->width - 20, -1);
}

G_MODULE_EXPORT void
on_button_new_test_clicked		(GtkObject       *object,
                                        gpointer         user_data)
{
	GtkBuilder *builder_local;
	GtkWidget *spinner;

	spinner = SpinnerWindow (_("Getting testplans. Please wait"));
	cur_file = 0;
	add_attach_num = 0;
	attach_num = 0;
	g_thread_create (pthread_cache, spinner, FALSE, NULL);
}


G_MODULE_EXPORT gboolean 
on_window1_key_press_event (GObject *obj, GdkEventKey *key, gpointer data)
{

	if (cur_state == 3)
	{
//		printf ("%d %d\n", key->state, key->keyval);
		if (key->state == 8 || key->state == 8192 || key->state == 8200 || key->state == 16 ||  key->state == 24 || key->state == 8216 || key->state == 8212 ||  key->state == 8208 ||  key->state == 8209)
		{
			switch (key->keyval)
			{
				/*1*/
				case 49:
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "radiobutton1")), TRUE);
				break;

				/*2*/
				case 50:
					gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gtk_builder_get_object (builder, "radiobutton2")), TRUE);
				break;
			}
		}
	}
	return FALSE;
}
