#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>

#include "support.h"
#include "shorthands.h"
#include "curfiletime.h"
#include "state.h"

#define TP_ID 1
#define TP_TITLE 2
#define TP_DESCR 3
#define TP_NAME 4

extern GtkBuilder *builder;

gchar global_tp_id[64];
gchar global_tp_name[64];

gboolean toggle_all 	(GtkTreeModel	*model,
                         GtkTreePath	*path,
                         GtkTreeIter 	*iter,
                         gpointer	data)
{
	gtk_list_store_set (GTK_LIST_STORE (model), iter, 0, 0, -1);	
	return FALSE;
}


static void
fixed_toggled		(GtkCellRendererToggle *cell,
		       	 gchar                 *path_str,
		       	 gpointer		data)
{
	GtkTreeModel *model = (GtkTreeModel *)data;
	GtkTreeIter  iter;
	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
	gchar *buf;
	

	/* get toggled iter */
	gtk_tree_model_get_iter (model, &iter, path);

	/* do something with the value */
	gtk_tree_model_foreach (data, toggle_all, 0);
	/* set new value */
	gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, TRUE, -1);

	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 1, &buf, -1);
	g_sprintf (global_tp_id, "%s", buf);

	write_cur_tp_id ();
	buf[0] = '\0';
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 4, &buf, -1);
	g_sprintf (global_tp_name, "%s", buf);

	/* clean up */
	gtk_tree_path_free (path);
	gtk_widget_set_sensitive (ForwardButton, TRUE);
//	g_free (buf);
}

void
add_columns	(GtkTreeView	*treeview, 
		 GtkListStore	*liststore)
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeModel *model; 
	GtkCellRendererToggle *toggle;
	gboolean radio;

	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (liststore));

    	renderer = gtk_cell_renderer_toggle_new ();

    	model =  gtk_tree_view_get_model (treeview);
    	g_signal_connect (renderer, "toggled",G_CALLBACK (fixed_toggled), model);

    	column = gtk_tree_view_column_new_with_attributes (_("Choose"), renderer, "active", 0, NULL);

	gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column), GTK_TREE_VIEW_COLUMN_FIXED);
	gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 50);
	gtk_tree_view_append_column (treeview, column);
    
 	gtk_cell_renderer_toggle_set_radio(GTK_CELL_RENDERER_TOGGLE(renderer),TRUE);

	renderer = gtk_cell_renderer_text_new ();
       
       	column = gtk_tree_view_column_new_with_attributes (_("title"), renderer, "text", TP_TITLE, NULL);
       	gtk_tree_view_column_set_resizable (column, TRUE);
       	gtk_tree_view_append_column (treeview, column);

       	renderer = gtk_cell_renderer_text_new ();
       	column = gtk_tree_view_column_new_with_attributes (_("description"), renderer, "text", TP_DESCR, NULL);
       	gtk_tree_view_append_column (treeview, column);
       	gtk_tree_view_column_set_resizable (column, TRUE);
}


xmlChar *
get_testplan_tag	(xmlNodePtr	cur, 
			gchar		*tagname)
{
	xmlChar *key;

	while (cur != NULL)
	{
		if (!xmlStrcmp(cur->name, (const xmlChar*)tagname))
		{
			key = xmlNodeGetContent (cur);
			return key;
		}
		cur = cur->next;
	}
	
	return NULL;
}


void
get_id_testplan		(xmlNodePtr	cur, 
			GtkListStore	*liststore, 
			GtkTreeIter 	iter)
{
	gtk_list_store_set (liststore, &iter, TP_ID, get_testplan_tag (cur, "id_tp"), -1);
}


void
get_descr_testplan	(xmlNodePtr	cur, 
			GtkListStore	*liststore, 
			GtkTreeIter 	iter)
{
	gtk_list_store_set (liststore, &iter, TP_DESCR, get_testplan_tag (cur, "description"), -1);
}

void
get_title_testplan	(xmlNodePtr	cur, 
			GtkListStore	*liststore, 
			GtkTreeIter 	iter)
{
	gtk_list_store_set (liststore, &iter, TP_TITLE, get_testplan_tag (cur, "title"), -1);
}

void
get_name_testplan	(xmlNodePtr	cur, 
			GtkListStore	*liststore, 
			GtkTreeIter 	iter)
{
	gtk_list_store_set (liststore, &iter, TP_NAME, get_testplan_tag (cur, "name"), -1);
}

void
get_testplans 		(xmlDocPtr	doc, 
			GtkListStore	*liststore, 
			GtkTreeIter	iter, 
			xmlNodePtr	cur) 
{
	xmlChar *key;
        cur = cur->xmlChildrenNode;
        xmlNodePtr cur_child;

        FILE *file;
        gchar filename[512];
        gint  filename_i = 0;

	get_id_testplan		(cur, liststore, iter);
	get_title_testplan 	(cur, liststore, iter);
	get_descr_testplan 	(cur, liststore, iter);
	get_name_testplan 	(cur, liststore, iter);
}


void
xml_testplan_read	(gchar *docname)
{
	GtkListStore *liststore;
	GtkTreeIter iter;
	GtkWidget *widget;
	xmlDocPtr doc;
        xmlNodePtr cur;

/*	liststore = gtk_list_store_new (5,
					G_TYPE_BOOLEAN,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING);

	widget = GTK_WIDGET (gtk_builder_get_object (builder,"treeview1"));
	add_columns (GTK_TREE_VIEW (widget), liststore);
*/
	widget = GTK_WIDGET (gtk_builder_get_object (builder,"treeview1"));

	if (cur_state == 4)
	{
		liststore = (GtkListStore*)gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
		if (liststore == NULL)
		{
			goto elselabel;
		}
		gtk_list_store_clear (liststore);
	}
	else
	{
elselabel:
		liststore = gtk_list_store_new (5,
					G_TYPE_BOOLEAN,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING,
					G_TYPE_STRING);
		add_columns (GTK_TREE_VIEW (widget), liststore);
	}

        doc = xmlParseFile(docname);
        if (doc == NULL ) 
        {
                return;
        }

        cur = xmlDocGetRootElement(doc);
        if (cur == NULL) 
        {
	        xmlFreeDoc(doc);
                return;
        }

        cur = cur->xmlChildrenNode;
        while (cur != NULL) 
        {
	        if ( xmlStrcmp (cur->name, (const xmlChar *)"testplan") == 0)
	        {
	    		gtk_list_store_append (liststore, &iter);
			get_testplans (doc, liststore, iter, cur);
		}
	        cur = cur->next;
        }
        xmlFreeDoc(doc);

	gtk_widget_show (widget);
}
