#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>
#include <glib.h>

#include "question.h"
#include "attach.h"
#include "check.h"
#include "support.h"

#define MY_ENCODING "UTF-8"

extern gint cur_file;
extern gchar global_tp_id[64];

gint
get_attribute (xmlNodePtr cur, char *attr_name, char *out)
{
	xmlChar *key = NULL;
  
	key= xmlGetNoNsProp (cur, (const xmlChar *)attr_name);

	if (key != NULL)
	{
        	g_sprintf (out, "%s", key);
		xmlFree (key);		
		return 1;
	}

	xmlFree (key);
	return 0;
}


void
parse_testplan (xmlDocPtr doc, xmlNodePtr cur) 
{
	xmlChar *key;
        cur = cur->xmlChildrenNode;

        while (cur != NULL) 
        {
    		if ((!xmlStrcmp(cur->name, (const xmlChar *)"keyword"))) 
    		{
            		key = xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
                        printf("keyword: %s\n", key);
                        xmlFree(key);
                }
        	cur = cur->next;
        }

        return;
}


xmlChar *
get_question_tag (xmlNodePtr cur, gchar *tagname)
{
	xmlChar *key;

	while (1)
	{
		if (!xmlStrcmp(cur->name, (const xmlChar*)tagname))
		{
			key = xmlNodeGetContent (cur);
			break;
		}
		cur = cur->next;
	}
	
	return key;
}


void
get_question_id (xmlNodePtr cur, FILE *file)
{
	g_fprintf (file, "[ID$]: %s\n", get_question_tag (cur, "id"));
}


void
get_question_name (xmlNodePtr cur, FILE *file)
{
	g_fprintf (file, "[NAME$]: %s\n", get_question_tag (cur, "name"));
}


void
get_question_article (xmlNodePtr cur, FILE *file)
{
	g_fprintf (file, "[ARTICLE$]: %s\n", get_question_tag (cur, "article"));
}


void
get_question_descr (xmlNodePtr cur, FILE *file)
{
	g_fprintf (file, "[DESCR$]: %s\n", get_question_tag (cur, "question_descr"));
}


void
get_question_answer (xmlNodePtr cur, FILE *file)
{
	g_fprintf (file, "[ANSWER$]: %s\n", get_question_tag (cur, "answer"));
}


void
get_question_comment (xmlNodePtr cur, FILE *file)
{
	g_fprintf (file, "[COMMENT$]: %s\n", get_question_tag (cur, "comment"));
}


void
get_question_attach (xmlNodePtr cur, FILE *file)
{
	g_fprintf (file, "[ATTACHMENT$]: %s\n", get_question_tag (cur, "attachment"));
}


int
get_attachable_attr (xmlNodePtr cur, FILE *file)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "attachable", str);
	if (rc)
	{
		if (!g_strcmp0 (str, "t"))
		{
			g_fprintf (file, "[ATTACHABLE$]: 1\n");
			return 1;
		}
		else
		{
			g_fprintf (file, "[ATTACHABLE$]: 0\n");
		}
	}
	else
	{
		g_fprintf (file, "[ATTACHABLE$]: 0\n");
	}
	return 0;
}


void
get_attachbale_num_attr (xmlNodePtr cur, FILE *file)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "attachable_num", str);
	if (rc)
	{
		g_fprintf (file, "[ATTACHABLE_NUM$]: %s\n", str);
	}
	else
	{
		g_fprintf (file, "[ATTACHABLE_NUM$]: 1\n");
	}
}

void
get_attachbale_size_attr (xmlNodePtr cur, FILE *file)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "attachable_size", str);
	if (rc)
	{
		g_fprintf (file, "[ATTACHABLE_SIZE$]: %s\n", str);
	}
	else
	{
		g_fprintf (file, "[ATTACHABLE_SIZE$]: 10000000\n");
	}
}


int
get_screenshot_attr (xmlNodePtr cur, FILE *file)
{
	gchar str[20];
	gint rc;

	rc = get_attribute (cur, "screenshot", str);
	if (rc)
	{
		if (!g_strcmp0 (str, "t"))
		{
			g_fprintf (file, "[SCREENSHOT$]: 1\n");
			return 1;
		}
		else
		{
			g_fprintf (file, "[SCREENSHOT$]: 0\n");
		}
	}
	else
	{
		g_fprintf (file, "[SCREENSHOT$]: 0\n");
	}
	return 0;
}

void
get_screenshots_num_attr (xmlNodePtr cur, FILE *file)
{
	gchar str[20];
	gint rc;

	rc =  get_attribute (cur, "screenshot_num", str);
	if (rc)
	{
		g_fprintf (file, "[SCREENSHOT_NUM$]: %s\n", str);
	}
	else
	{
		g_fprintf (file, "[SCREENSHOT_NUM$]: 5\n");
	}
}


void
get_screenshots_size_attr (xmlNodePtr cur, FILE *file)
{
	gchar str[20];
	gint rc;

	rc =  get_attribute (cur, "screenshot_size", str);
	if (rc)
	{
		g_fprintf (file, "[SCREENSHOT_SIZE$]: %s\n", str);
	}
	else
	{
		g_fprintf (file, "[SCREENSHOT_SIZE$]: 1000000\n");
	}
}

void
get_questions (xmlDocPtr doc, xmlNodePtr cur) 
{
	xmlChar *key;
        cur = cur->xmlChildrenNode;
        xmlNodePtr cur_child;
        GtkTreeIter iter;
        FILE *file;
        gchar filename[512];
        gint  filename_i = 0, rc;

        while (cur != NULL)
        {
    		if ((!xmlStrcmp(cur->name, (const xmlChar *)"question"))) 
		{
			cur_child = cur->xmlChildrenNode;
			g_sprintf (filename, "%s/%s/store/%d", g_get_home_dir() , CACHE_DIR, filename_i++);
			file = fopen (filename, "w+");
			g_fprintf 		(file, "[ANSWERED$]: 0\n");
			get_question_id 	(cur_child, file);
			get_question_article 	(cur_child, file);
			get_question_name 	(cur_child, file);
			get_question_descr 	(cur_child, file);
			rc = get_attachable_attr (cur, file);
			if (rc)
			{
				get_attachbale_num_attr (cur, file);
				get_attachbale_size_attr (cur, file);
			}
			
			rc = get_screenshot_attr (cur, file);
			if (rc)
			{
				get_screenshots_num_attr (cur, file);
				get_screenshots_size_attr (cur, file);
			}
			
			g_fprintf (file, "[CUR_ATTACH$]: 0\n");
			questions_count ++;
			fclose (file);
		}
    		cur = cur->next;
    }
}


void
read_xml (gchar *docname, GtkListStore *liststore)
{
	xmlDocPtr doc;
        xmlNodePtr cur;

        doc = xmlParseFile(docname);
        if (doc == NULL ) 
        {
                return;
        }

        cur = xmlDocGetRootElement(doc);
        if (cur == NULL) 
        {
	        xmlFreeDoc(doc);
                return;
        }

        cur = cur->xmlChildrenNode;
        while (cur != NULL) 
        {
	        if ( xmlStrcmp (cur->name, (const xmlChar *)"questions" ) == 0)
	        {
			get_questions (doc, cur);
		}
	        cur = cur->next;
        }
        xmlFreeDoc(doc);
	write_questions_count ();
}


xmlChar *
ConvertInput(const gchar *in, const gchar *encoding)
{
    xmlChar *out;
    gint ret;
    gint size;
    gint out_size;
    gint temp;
    xmlCharEncodingHandlerPtr handler;

    if (in == 0)
        return 0;

    handler = xmlFindCharEncodingHandler(encoding);

    if (!handler) {
        printf("ConvertInput: no encoding handler found for '%s'\n",
               encoding ? encoding : "");
        return 0;
    }

    size = (int) strlen(in) + 1;
    out_size = size * 2 - 1;
    out = (unsigned char *) xmlMalloc((size_t) out_size);

    if (out != 0) {
        temp = size - 1;
        ret = handler->input(out, &out_size, (const xmlChar *) in, &temp);
        if ((ret < 0) || (temp - size + 1)) {
            if (ret < 0) {
                printf("ConvertInput: conversion wasn't successful.\n");
            } else {
                printf
                    ("ConvertInput: conversion wasn't successful. converted: %i octets.\n",
                     temp);
            }

            xmlFree(out);
            out = 0;
        } else {
            out = (unsigned char *) xmlRealloc(out, out_size + 1);
            out[out_size] = 0;  /*null terminating out */
        }
    } else {
        printf("ConvertInput: no mem\n");
    }

    return out;
}


void
xml_generate_report ()
{
	gint rc, i, last_file = 0;
	gchar filename[512], system_id[64]="\0", fa_name [256];;
	gchar q_id[64] = "", q_article[32] = "", q_answer[8] = "", q_comment[4096] = "", q_attach[512] = "", q_attach_fullname[512] = "";
	xmlTextWriterPtr writer;
	xmlChar *tmp;
	FILE *f_attachment, *fa;

	g_sprintf (filename, "%s/%s/report.xml", g_get_home_dir (), CACHE_DIR);
	g_sprintf (fa_name,"%s/%s/attachs", g_get_home_dir (), CACHE_DIR);


	
	writer = xmlNewTextWriterFilename (filename, 0);

	rc = xmlTextWriterStartDocument (writer, NULL, MY_ENCODING, NULL);
	if (rc < 0)
	{
		return;
	}

	rc = xmlTextWriterStartElement (writer, BAD_CAST "testplan");
	if (rc < 0)
	{
		return;
	}



	choppy (global_tp_id, '\0');
	tmp = ConvertInput (global_tp_id, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "id_tp", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput ((gchar*)get_time_testplan_getted (), MY_ENCODING);
	tmp[strlen(tmp)-1] = '\0';
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_testplan_getted", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput ((gchar*)get_time_test_started (), MY_ENCODING);
	tmp[strlen(tmp)-1] = '\0';
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_test_started", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput ((gchar*)get_time_test_finished (), MY_ENCODING);
	tmp[strlen(tmp)-1] = '\0';
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_test_finished", tmp);
	if (rc < 0)
	{
		return;
	}

	tmp = ConvertInput ((gchar*)get_time_report_send (), MY_ENCODING);
	tmp[strlen(tmp)-1] = '\0';
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "time_report_send", tmp);
	if (rc < 0)
	{
		return;
	}

	get_hw_id (system_id);
	tmp[strlen(tmp)-1] = '\0';
	tmp = ConvertInput ((gchar*)system_id, MY_ENCODING);
	rc = xmlTextWriterWriteElement(writer, BAD_CAST "system_id", tmp);
	if (rc < 0)
	{
		return;
	}

	rc = xmlTextWriterStartElement(writer, BAD_CAST "questions");
        if (rc < 0) 
        {
            return;
        }

	fa = fopen (fa_name, "w+");

	last_file = get_last_file ();
//	printf ("%d\n", last_file);
        for (i = 0; i <= last_file; i++)
        {
    		rc = xmlTextWriterStartElement(writer, BAD_CAST "question");
    		read_question (&q_id, &q_article, &q_answer, &q_comment, &q_attach, i);
		f_attachment = open_for_attachment (i);

		if (!f_attachment)
		{
		}
    		bzero (q_attach, 512);
		bzero (q_attach_fullname, 512);

	        if (rc < 0) 
    		{
        	    return;
    		}

    		tmp = ConvertInput (q_id, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "id", tmp);
		if (rc < 0)
		{
		//	error TextWriterStartElement
			return;
		}
    		tmp = ConvertInput (q_article, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "article", tmp);
		if (rc < 0)
		{
		//	error TextWriterStartElement
			return;
		}

    		tmp = ConvertInput (q_answer, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "answer", tmp);
		if (rc < 0)
		{
		//	error TextWriterStartElement
			return;
		}

    		tmp = ConvertInput (q_comment, MY_ENCODING);
		rc = xmlTextWriterWriteElement(writer, BAD_CAST "comment", tmp);
		if (rc < 0)
		{
		//	error TextWriterStartElement
			return;
		}

		rc = xmlTextWriterStartElement(writer, BAD_CAST "attachments");
	        if (rc < 0) 
	        {
			return;
        	}

		while (1)
		{
			if (!get_attachment (f_attachment, q_attach, q_attach_fullname))
			{
				close_attachment (f_attachment);
				break;
			}
			else
			{
				g_fprintf (fa,"%s", q_attach_fullname);
				choppy (q_attach, '\0');
				tmp = ConvertInput (q_attach, MY_ENCODING);
				rc = xmlTextWriterWriteElement(writer, BAD_CAST "attachment", tmp);

				if (rc < 0)
				{
					fclose (fa);
					return;
				}
			}
		}

		rc = xmlTextWriterEndElement(writer);

		bzero (q_id, 64);
		bzero (q_article, 32);
		bzero (q_comment, 4096);
		bzero (q_answer, 8);
    		rc = xmlTextWriterEndElement(writer);

        }

	rc = xmlTextWriterEndDocument(writer);

	if (rc < 0) 
	{
		fclose (fa);
		return;
	}

	fclose (fa);
	xmlFreeTextWriter(writer);
	xmlFree (tmp);
}
